import json
from .utils import to_csv  # Import the to_csv function from the utils module
from langchain_openai import ChatOpenAI
from langchain.prompts import PromptTemplate
from langchain.chains import LLMChain
from langchain.output_parsers import PydanticOutputParser
from .models import Quiz
    
def generate_mcq(topic, level, num = 1, file_name=None, model='gpt-3.5-turbo', temperature=0.7):

    parser = PydanticOutputParser(pydantic_object=Quiz)
    
    llm = ChatOpenAI(model=model, temperature=temperature)

    format_instructions = parser.get_format_instructions()

    MCQ_template = """
    Generate {num} multiple-choice question (MCQ) based on the given topic and level.
    provide the question, four answer options, and the correct answer.

    Topic: {topic}
    Level: {level}

    The response should be in JSON format. \n {format_instructions}
    """

    MCQ_prompt = PromptTemplate(input_variables=["num", "topic", "level"],
                                template=MCQ_template,
                                partial_variables={"format_instructions": format_instructions}
                                )

    MCQ_chain = LLMChain(llm=llm, prompt=MCQ_prompt)
    results = MCQ_chain.invoke({"num": num,
                                    "topic": topic,
                                    "level": level
                                    }, return_only_outputs=True)

    results = results["text"]
    structured_output = parser.parse(results)
    
    # Generate Csv file if file_name is provided
    if file_name:
        to_csv(structured_output, file_name)
        
    return structured_output



from typing import List
from langchain_core.pydantic_v1 import BaseModel

class Question(BaseModel):
    question: str
    options: List[str]
    correct_answer: str

class Quiz(BaseModel):
    questions: List[Question]


from langchain_openai import ChatOpenAI
from langchain.prompts import PromptTemplate
from langchain.chains import LLMChain

def generate_lesson_plan(subject, level, llm=ChatOpenAI()):
    prompt = PromptTemplate(
        input_variables=["subject", "level"],
        template="""
    Generate a lesson plan for the given subject and level.
    Provide a detailed outline of the lesson, including an introduction, main activities, and a conclusion.

    Subject: {subject}
    Level: {level}

    Lesson Plan:
    """,
    )

    # Create an LLMChain using the prompt and ChatOpenAI
    lesson_plan_chain = LLMChain(llm=llm, prompt=prompt)

    # Generate the lesson plan
    lesson_plan = lesson_plan_chain.predict(subject=subject, level=level)

    return lesson_plan


import pandas as pd
from .models import Quiz

def to_csv(quiz_data : Quiz, file_name):
    """
    Generate a CSV file from a Quiz object.

    Args:
    quiz_data (Quiz): Instance of the Quiz class containing a list of Question objects.
    file_name (str): Name of the CSV file to be created.
    """
    mcq_data = []

    for question in quiz_data.questions:
        mcq_data.append({
            'question': question.question,
            'options': ', '.join(question.options),
            'correct_answer': question.correct_answer
        })

    df = pd.DataFrame(mcq_data)
    df.to_csv(file_name, index=False)